/*
   (C) 2018 R.Schick / Thorsten Gurzan - beelogger.de

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

// beelogger.de - Arduino Datenlogger für Imker
// Erläuterungen dieses Programmcodes unter http://beelogger.de

#include <Wire.h>

//----------------------------------------------------------------
// Konfiguration
//----------------------------------------------------------------

#define Anzahl_Sensoren_BME280  2 // Mögliche Werte: '0','1','2'

#define Korrektur_Luftdruck 0.0 // Korrekturwert um Abweichungen zu offiziellen Wetterstationen auszugleichen

//----------------------------------------------------------------
// Ende Konfiguration
//----------------------------------------------------------------


//----------------------------------------------------------------
// Parameter BME280 - Temperatur und Luftfeuchte
//----------------------------------------------------------------

#include <beelogger_Adafruit_BME280.h>

uint8_t BME280_adresse[2] = {0x76, 0x77};  // I2C-Adresse BM280 Nr.1, 2

#define SDA   A4      // I2C Daten
#define SCL   A5      // I2C Clock

#define SEALEVELPRESSURE_HPA (1013.25f)
//----------------------------------------------------------------


const float No_Val = 999.99;
float Temp[2] = {No_Val, No_Val};
float Feuchte[2] = {No_Val, No_Val};
float L_Druck[2] = {No_Val, No_Val};

// Stromversorgung für Sensoren, Module und DS3231
#ifdef ARDUINO_ARCH_AVR
#define Power_Pin 4

#else
#define Power_Pin PB5   // for STM32Fxyz
#endif


void setup() {
  Serial.begin(9600);
  while (!Serial) {};
  Serial.println("BME280 Test  15.04.2023");
  Serial.println(" ");
  digitalWrite(Power_Pin, HIGH);
  pinMode(Power_Pin, OUTPUT);
  delay(1000);
}

void loop() {

  Serial.println(F("BME280- hole Werte: "));
  delay(500);
  Sensor_BME280();
  for (byte i = 0 ; i < Anzahl_Sensoren_BME280; i++) {
    Serial.print(F("BME280 Nr.: "));
    Serial.println(i + 1);
    Serial.print(F("BME280- I2C - Adresse:  0x"));
    Serial.println(BME280_adresse[i], HEX);

    if (Temp[i] == No_Val)  {
      Serial.println(F("BME280- Fehler "));
    }
    else {
      Serial.print(F("Temperatur: "));
      Serial.println(Temp[i]);
      Serial.print(F("Luftfeuchte: "));
      Serial.println(Feuchte[i]);
      Serial.print(F("Luftdruck: "));
      L_Druck[i] = L_Druck[i] / 100.0;
      L_Druck[i]  = L_Druck[i] + Korrektur_Luftdruck;
      Serial.println(L_Druck[i]);
    }
    Serial.flush();
  }
  delay(2000);
}



//----------------------------------------------------------------
// Funktion BME280 - Temperatur und Luftfeuchte
//----------------------------------------------------------------
void Sensor_BME280() {
  if (Anzahl_Sensoren_BME280 > 0) {
    float Temperatur_BME;
    float Luftfeuchte_BME;
    float Luftdruck_BME;
    boolean check;

    beelogger_Adafruit_BME280 my_bme;

    for (byte i = 0 ; i < Anzahl_Sensoren_BME280; i++) {
      check = my_bme.begin(BME280_adresse[i]); // I2C Adresse
      delay (100); // time to get system ready
      if (check) {   // if bme ok
        Temperatur_BME = my_bme.readTemperature();
        Luftfeuchte_BME = my_bme.readHumidity();
        Luftdruck_BME = my_bme.readPressure();
      }
      else {
        Temperatur_BME = No_Val;
        Luftfeuchte_BME = No_Val;
        Luftdruck_BME = No_Val;
      }
      if (i == 0) { // erster BME
        Temp[0] = Temperatur_BME;      // Hier kann die Zuordnung der Sensoren geändert werden
        Feuchte[0] = Luftfeuchte_BME;     // Hier kann die Zuordnung der Sensoren geändert werden
        L_Druck[0] = Luftdruck_BME;
      }
      if (i == 1) { // zweiter BME
        Temp[1] = Temperatur_BME;      // Hier kann die Zuordnung der Sensoren geändert werden
        Feuchte[1] = Luftfeuchte_BME;     // Hier kann die Zuordnung der Sensoren geändert werden
        L_Druck[1] = Luftdruck_BME;
      }
    }
  }
}
//----------------------------------------------------------------
