/*
   (C) 2022 R.Schick - beelogger.de

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

// beelogger.de - Arduino Datenlogger für Imker
// Erläuterungen dieses Programmcodes unter http://beelogger.de
// Version 2023



//----------------------------------------------------------------
// Konfiguration
//----------------------------------------------------------------

#define Anzahl_Sensoren_DHT 1     // Mögliche Anzahl: '0','1','2'  --- Nr 1,2 ---  (Temperatur + Luftfeuchte)


#ifdef ARDUINO_ARCH_AVR
#define DHT_Nr_1_Pin 5
#define DHT_Nr_2_Pin 6

#else  // for STM32Fxyz
#define DHT_Nr_1_Pin PB12
#define DHT_Nr_2_Pin PB13
#endif

#define DHT_LIB_Typ 1  // 0 = Rob Tillart dhtnew-lib,  1 =  beelogger Version of Adafruit DHT-lib nur für ATmega

//----------------------------------------------------------------
// Ende Konfiguration
//----------------------------------------------------------------


#ifdef ARDUINO_ARCH_AVR  // ATMega
#include "LowPower.h"
#if (DHT_LIB_Typ == 0)
#include <dht.h>
#else
#include <beelogger_DHT.h>  //beelogger Version of Adafruit DHT-Lib
#endif
#else  // STM32
#include <dhtnew.h>
#if (DHT_LIB_Typ == 1)
#undef DHT_LIB_Typ
#define DHT_LIB_Typ 0  // don't change
#endif
#endif

uint8_t DHT_Sensor_Pin[2] = { DHT_Nr_1_Pin, DHT_Nr_2_Pin };

const float No_Val = 99.99;
const float No_Value = -1.0;

// Für beelogger-Solar: Aktivierung der Stromversorgung für Sensoren, Module und DS3231
#ifdef ARDUINO_ARCH_AVR  // ATMega
#define Power_Pin 4
#else
#define Power_Pin PB5  // for STM32Fxyz
#endif


void setup() {
  Serial.begin(9600);
  while (!Serial) {};
  Serial.println("DHT 21/22 AM2301/AM2302  Test  23.01.2023");
  Serial.println(" ");

#ifdef ARDUINO_ARCH_AVR  // ATMega
#if (DHT_LIB_Typ == 1)   // Adafruit DHTlib
  Serial.println(" beelogger Version Adafruit DHT-Lib,");
#else
  Serial.println("Rob Tillart dhtnew.lib ");
#endif
  Serial.println(" ");
#endif
  digitalWrite(Power_Pin, HIGH);
  pinMode(Power_Pin, OUTPUT);
  delay(1000);
}


void loop() {

  Sensor_DHT();

  Serial.println(" 4 sec. Power Off ...");
  Serial.flush();
  for (uint8_t i = 0; i < Anzahl_Sensoren_DHT; i++) {
    digitalWrite(DHT_Sensor_Pin[i], LOW);
    pinMode(DHT_Sensor_Pin[i], INPUT);
  }
  digitalWrite(Power_Pin, LOW);
#ifdef ARDUINO_ARCH_AVR
  LowPower.powerDown(SLEEP_2S, ADC_OFF, BOD_OFF);  //delay(2000);
  LowPower.powerDown(SLEEP_2S, ADC_OFF, BOD_OFF);  //delay(2000);
#else
  delay(4000);
#endif
  Serial.println("\n Power On ");
  digitalWrite(Power_Pin, HIGH);
}


#if DHT_LIB_Typ   // 1 = beelogger Version of Adafruit DHT-Lib, 0 = Rob Tillart DHTLib

#include <beelogger_DHT.h>  //Adafruit DHT-Lib

void Sensor_DHT() {

  float Temperatur_DHT;
  float Luftfeuchte_DHT;
  Serial.println(F("DHT:"));
  Serial.flush();

  for (uint8_t i = 0 ; i < Anzahl_Sensoren_DHT; i++) {
    beelogger_DHT dht(DHT_Sensor_Pin[i], 22);
    dht.begin();
    uint8_t x = 0;
    do {
      delay(2100);     // Wait a few seconds between measurements.
      Temperatur_DHT = dht.readTemperature();    // Read temperature as Celsius (the default)
      Luftfeuchte_DHT = dht.readHumidity();
      x--;
      if (x == 0) break;
      Serial.println("*");
    } while (((Temperatur_DHT == 1.0) && (Luftfeuchte_DHT == 1.0)) ||
             (Temperatur_DHT < -30.0) || (Luftfeuchte_DHT == 0.0));

    // Check if any reads failed
    if (isnan(Temperatur_DHT) || (Temperatur_DHT < -20.0) || (Temperatur_DHT > 60.0)) Temperatur_DHT = No_Val;
    if (isnan(Luftfeuchte_DHT) || (Luftfeuchte_DHT < 2.0) || (Luftfeuchte_DHT >= 99.9)) Luftfeuchte_DHT = No_Value;


    Serial.print(F(" [C]: "));
    Serial.println(Temperatur_DHT);
    Serial.print(F(" [%RH]: "));
    Serial.println(Luftfeuchte_DHT);
    Serial.flush();
  }
}

#else  // other Lib

#include <dhtnew.h>  // Rob Tillart DHTNEW Lib  0.2.0  2020-05-02

void Sensor_DHT() {

  float Temperatur_DHT;
  float Luftfeuchte_DHT;

  Serial.println(F("DHT:"));
  Serial.flush();
  for (uint8_t i = 0; i < Anzahl_Sensoren_DHT; i++) {
    Temperatur_DHT = No_Val;
    Luftfeuchte_DHT = No_Value;
    DHTNEW beeDHT(DHT_Sensor_Pin[i]);
    beeDHT.read();
    delay(2100); // wait for sensor ready, first readout maybe garbage
    beeDHT.read();
    Luftfeuchte_DHT = beeDHT.getHumidity();
    Temperatur_DHT = beeDHT.getTemperature();
    // Check if any reads failed
    if (isnan(Temperatur_DHT) || (Temperatur_DHT < -20.0) || (Temperatur_DHT > 60.0)) Temperatur_DHT = No_Val;
    if (isnan(Luftfeuchte_DHT) || (Luftfeuchte_DHT < 2.0) || (Luftfeuchte_DHT >= 99.9)) Luftfeuchte_DHT = No_Value;


    Serial.print(F(" [C]: "));
    Serial.println(Temperatur_DHT);
    Serial.print(F(" [%RH]: "));
    Serial.println(Luftfeuchte_DHT);
    Serial.flush();
  }
}
#endif
