/*
   (C) 2018 R.Schick / Thorsten Gurzan - beelogger.de

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

// beelogger.de - Arduino Datenlogger für Imker
// Erläuterungen dieses Programmcodes unter http://beelogger.de

#include <Wire.h>


//----------------------------------------------------------------
// Konfiguration Si7021 - Temperatur und Luftfeuchte
//----------------------------------------------------------------
#include "Adafruit_Si7021.h"
#include "SI7021.h"
// Si7021 I2C-Adresse : 0x40 (64d), Wert ist in Adafruit_Si7021.h
#define SDA   A4      // I2C Daten
#define SCL   A5      // I2C Clock
//----------------------------------------------------------------


const float No_Val = 999.99;
float TempIn = No_Val;
float FeuchteIn = No_Val;

// Stromversorgung für Sensoren, Module und DS3231
#ifdef ARDUINO_ARCH_AVR
#define Power_Pin 4

#else
#define Power_Pin PB5   // for STM32Fxyz
#endif


void setup() {
  Serial.begin(9600);
  while (!Serial) {};
  Serial.println("Si7021 Test  11.03.2022");
  Serial.println(" ");
  digitalWrite(Power_Pin, HIGH);
  pinMode(Power_Pin, OUTPUT);
  delay(1000);
}

void loop() {

  Serial.println(F("Si7021- hole Werte: "));
  delay(500);
  Sensor_Si7021_Ada();
  Sensor_Si7021();
  Serial.flush();

  delay(10000);
}



//----------------------------------------------------------------
// Funktion Si7021 - Temperatur und Luftfeuchte //RS
// mit Adafruit Biblothek
//----------------------------------------------------------------
void Sensor_Si7021_Ada() {
  float Temperatur_SI;
  float Luftfeuchte_SI;

  Adafruit_Si7021 mySI7021 = Adafruit_Si7021();

  int check = mySI7021.begin();
  Serial.print("Adafruit- Bibliothek:  ");
  Serial.println(check);

  if (check) {
    Luftfeuchte_SI = mySI7021.readHumidity();
    Temperatur_SI = mySI7021.readTemperature();
    Serial.print(F("Si7021- Temperatur: "));
    Serial.println(Temperatur_SI);
    Serial.print(F("Si7021- Feuchte: "));
    Serial.println(Luftfeuchte_SI);
  }
  else {
    Serial.println(F("  Si7021- Fehler "));
    Luftfeuchte_SI = No_Val;
    Temperatur_SI = No_Val;
  }
  Serial.println(" ");
}
//----------------------------------------------------------------
// Funktion Si7021 - Temperatur und Luftfeuchte //RS
// mit Sorensen Biblothek
//----------------------------------------------------------------
void Sensor_Si7021() {
  float Temperatur_SI;
  float Luftfeuchte_SI;

  SI7021 sensor7021;
  int check = sensor7021.begin();

  Serial.print("Sorensen- Bibliothek:  ");
  Serial.println(check);

  if (check) {
    int temperature = sensor7021.getCelsiusHundredths();
    Temperatur_SI = ((float) temperature) / 100.0;
    int humidity = sensor7021.getHumidityPercent();
    Luftfeuchte_SI = ((float) humidity );
    Serial.print(F("Si7021- Temperatur: "));
    Serial.println(Temperatur_SI);
    Serial.print(F("Si7021- Feuchte: "));
    Serial.println(Luftfeuchte_SI);
  }
  else {
    Serial.println(F("  Si7021- Fehler "));
    Luftfeuchte_SI = No_Val;
    Temperatur_SI = No_Val;
  }
  Serial.println(" ");
}
//----------------------------------------------------------------
